App.controller('Controller', function ($scope, $location) {
    var _this = this;
    _this.SerialNumber = '#########';

    _this.Connected = false;
    _this.Config = { TerminationString: "\\r\\n", ReportSendIntervalInMinutes: 60 };
    _this.SavedConfig = angular.copy(_this.Config);
    _this.SelectedType = -1;


    // instantiate our JniorConfigStorage class and pass in our websocket object
    var jniorConfigStorage = new JniorConfigStorage(jnrWebsocket, '/flash/wedauto/flourflow/config.json');

    // onLoggedIn callback.  this is where we will read the config file
    jnrWebsocket.addOnLoggedInListener(function onLoggedIn() {
        console.log('onLoggedIn');
        _this.readConfig();

        jnrWebsocket.mkDir("/flash/wedauto", function (json) {
            if ("Succeed" == json.Result) {
                jnrWebsocket.mkDir("/flash/wedauto/flourflow", function (json) {
                });
            }
        });

        jnrWebsocket.readRegistryKeys(
            ["$serialnumber", "ipconfig/hostname"],
            function (key, value) {
                if (key.endsWith('serialnumber')) _this.SerialNumber = value;
                else if (key.endsWith('hostname')) _this.HostName = value;

                $scope.$apply();
            }
        );

        //
        // regisyter a listener event
        jnrWebsocket.addEventListener('onMessage', _this.onMessage);

        //
        // get the application information and the curent state
        jnrWebsocket.postMessage(APP_ID, { Message: 'application.info.get' });
        jnrWebsocket.postMessage(APP_ID, { Message: 'state.get' });

        $scope.$apply();
    });



    jnrWebsocket.onClosed = function () {
        _this.Connected = false;

        $scope.$apply();
    };



    _this.hasChanged = function (config, savedConfig) {
        var changed = !angular.equals(config, savedConfig);
        return changed;
    };



    _this.readConfig = function () {
        jniorConfigStorage.readConfig(function (result, config) {
            if ('Fail' !== result) {
                _this.Config = config;

                _this.SavedConfig = angular.copy(_this.Config);
                console.log('Config: ' + _this.Config);

                $scope.$apply();
            }
        });
    };


    _this.validate = function () {
        //
        // go through each type host.  make sure that the model has enough inputs for the
        // configured starting input + input count

        if (!$scope.Form.$valid) {
            bootbox.alert({
                className: 'bb-danger',
                title: 'Errors detected',
                message: 'There are errors.  Please fix the errors and try again.'
            });
            return false;
        }
        return true;
    };


    _this.saveChangesRequest = function () {
        if (!_this.validate()) return;

        _this.SaveDialog = bootbox.confirm({
            className: "bb-primary",
            title: "Are you sure you want to save your changes?",
            message: "You will need to reboot for the changes to take affect.",
            buttons: {
                cancel: {
                    label: 'Cancel',
                    className: 'btn-default'
                },
                confirm: {
                    label: 'Save',
                    className: 'btn-primary'
                }
            },
            callback: function (result) {
                if (result) {
                    _this.saveChanges();
                    return false;
                }
            }
        });
        _this.SaveDialog.init(function () {
            $("#bootbox-confirm-btn").prop('disabled', false);
            $("#bootbox-cancel-btn").prop('disabled', false);
        });

    };


    _this.saveChanges = function () {
        jniorConfigStorage.Config = _this.Config;
        jniorConfigStorage.saveConfig(function (status) {
            if ('Fail' !== status) {
                _this.SavedConfig = angular.copy(_this.Config);
                $scope.$apply();

                jnrWebsocket.postMessage(APP_ID, { Message: 'config.updated' });

                _this.SaveDialog.modal('hide');
            } else {
                alert('error saving the file');
            }
        });
    };


    _this.cancelChangesRequest = function () {
        _this.CancelDialog = bootbox.confirm({
            className: 'bb-warning',
            title: "Are you sure you want to cancel your changes?",
            message: "Cancelling your changes will reload the last saved settings",
            buttons: {
                cancel: {
                    label: 'Keep Changes',
                    className: 'btn-default'
                },
                confirm: {
                    label: 'Cancel Changes',
                    className: 'btn-warning'
                }
            },
            callback: function (result) {
                if (result) {
                    _this.cancelChanges();
                    return false;
                }
            }
        });
        _this.CancelDialog.init(function () {
            $("#bootbox-confirm-btn").prop('disabled', false);
            $("#bootbox-cancel-btn").prop('disabled', false);
        });
    };


    _this.cancelChanges = function () {
        _this.Config = angular.copy(_this.SavedConfig);
        $scope.$apply();

        _this.CancelDialog.modal('hide');
    };



    _this.sendReboot = function () {
        jnrWebsocket.postMessage(APP_ID, { Message: 'reboot.execute' });
    };



    _this.onMessage = function (json) {
        var message = json.Message;

        if ('application.info.update' === message) {
            _this.Connected = true;

            $scope.$apply();

        } else if ('state.update' === message) {
            _this.Connected = true;

            _this.TransmissionSession = json.TransmissionSession;

            $scope.$apply();
        }
    };




    function array_move(arr, old_index, new_index) {
        if (new_index >= arr.length) {
            var k = new_index - arr.length + 1;
            while (k--) {
                arr.push(undefined);
            }
        }
        arr.splice(new_index, 0, arr.splice(old_index, 1)[0]);
        return arr; // for testing
    };



    _this.addSourceHost = function () {
        if (!_this.Config.SourceHosts) _this.Config.SourceHosts = [];
        _this.Config.SourceHosts.push({});
    };



    _this.moveSourceHostUp = function (typeHost) {
        var sourceHostIndex = _this.Config.SourceHosts.indexOf(typeHost);
        if (-1 != sourceHostIndex) {
            array_move(_this.Config.SourceHosts, sourceHostIndex, sourceHostIndex - 1);
        }
    };



    _this.moveSourceHostDown = function (typeHost) {
        var sourceHostIndex = _this.Config.SourceHosts.indexOf(typeHost);
        if (-1 != sourceHostIndex) {
            array_move(_this.Config.SourceHosts, sourceHostIndex, sourceHostIndex + 1);
        }
    };



    _this.removeSourceHost = function (typeHost) {
        var sourceHostIndex = _this.Config.SourceHosts.indexOf(typeHost);
        if (-1 != sourceHostIndex) {
            _this.Config.SourceHosts.splice(sourceHostIndex, 1)
        }
    };



    _this.addDestinationHost = function () {
        if (!_this.Config.DestinationHosts) _this.Config.DestinationHosts = [];
        _this.Config.DestinationHosts.push({});
    };



    _this.moveDestinationHostUp = function (typeHost) {
        var destinationHostIndex = _this.Config.DestinationHosts.indexOf(typeHost);
        if (-1 != destinationHostIndex) {
            array_move(_this.Config.DestinationHosts, destinationHostIndex, destinationHostIndex - 1);
        }
    };



    _this.moveDestinationHostDown = function (typeHost) {
        var destinationHostIndex = _this.Config.DestinationHosts.indexOf(typeHost);
        if (-1 != destinationHostIndex) {
            array_move(_this.Config.DestinationHosts, destinationHostIndex, destinationHostIndex + 1);
        }
    };



    _this.removeDestinationHost = function (typeHost) {
        var destinationHostIndex = _this.Config.DestinationHosts.indexOf(typeHost);
        if (-1 != destinationHostIndex) {
            _this.Config.DestinationHosts.splice(destinationHostIndex, 1)
        }
    };
});